::cisco::eem::event_register_timer watchdog name composite_device_health_interface time $interface_health_period queue_priority low nice 1 maxrun 600

###############################################################################################################
#
#  Revision #          :  1.8
#  Last Updated        :  September 18, 2007
#  Author/Contributor  :  David Lin, dalin@cisco.com
#
#  Description         :  This EEM Tcl script can be used as part of an overall strategy to monitor basic device 
#                         health in the network, namely the handling of common interface errors which include
#                         input errors, output errors, crc errors, interface resets and congested input queue.  
#                         This Tcl script monitors the health of physical interfaces. 
#                         If subinterfaces are present, this Tcl script requires that the logical interface(s) 
#                         conform to the following format:
#
#                         <interface><slot#>/<port#>.<vlan#>
#
#                         Example:
#                                  GigabitEthernet0/0.1  (supported)
#                                  GigabitEthernet0.1    (not supported)
#                         
#                         Note also that because subinterfaces are logical, interface errors for the 
#                         physical interface for which the subinterface(s) belong to can only be monitored.
#
#                         Finally, the user may modify the subinterface conformance restriction as desired if 
#                         say the number of slots differs from the above example. 
#
#                         Search for the following strings to goto the appropriate section of the code to make
#                         your changes:
#
#                    if [catch {cli_exec $cli(fd) "show ip int brief | inc \[0-9\]*/\[0-9\]*\\.\[0-9\]*"} result] {
#                         error $result $errorInfo
#                    }
#                    set interface $result
#
#
#
#  Requirements        :  -EEM env variables-
#                         event manager environment input_errors_threshold <#>
#                         event manager environment output_errors_threshold <#>
#                         event manager environment crc_threshold <#>
#                         event manager environment interface_reset_threshold <#>
#                         event manager environment interface_health_period <secs>
#
#                         Example:   event manager environment input_errors_threshold 1000
#                                    event manager environment output_errors_threshold 1000
#                                    event manager environment crc_threshold 1000
#                                    event manager environment interface_reset_threshold 5
#                                    event manager environment interface_health_period 3600
#    
#                         event manager environment _email_server <your-mailserver-ipaddress or dns-name>
#                         event manager envrionment _email_from <your-email-from-address>
#                         event manager environment _email_to <your-email-to-address>
#
#
#                         Example: event manager environment _email_server 10.10.10.10
#                                  event manager environment _email_from router-123@cisco.com
#                                  event manager environment _email_to noc@cisco.com
#
#
#                         -EEM trigger-
#                         This script uses a watchdog timer and will run every $interface_health_period
#                         as soon as the script is registered.
#
#                         -EEM action-
#                         Sends an email message to specified server defined in variable _email_server above
#
#  Cisco Products tested :   c2821
#
#  Cisco IOS Version tested :   12.4(9)T4
#
###############################################################################################################

#
# Namespace imports
#
namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

#--- Check required environment variable(s) has been defined

if {![info exists input_errors_threshold]} {
    set result "EEM Policy Error: variable input_errors_threshold has not been set"
    error $result $errorInfo
}

if {![info exists output_errors_threshold]} {
    set result "EEM Policy Error: variable output_errors_threshold has not been set"
    error $result $errorInfo
}

if {![info exists crc_threshold]} {
    set result "EEM Policy Error: variable crc_threshold has not been set"
    error $result $errorInfo
}

if {![info exists interface_reset_threshold]} {
    set result "EEM Policy Error: variable interface_reset_threshold has not been set"
    error $result $errorInfo
}

if {![info exists interface_health_period]} {
    set result "EEM Policy Error: variable interface_health_period has not been set"
    error $result $errorInfo
}

#------------------ hostname  ----------------------
set routername [info hostname]


#
#-------------------   " cli open"   -------------------
#
if [catch {cli_open} result] {
  error $result $errorInfo
} else {
  array set cli $result
}

#--------------- end of  "cli open"   -------------------

#
#----------------------- "show commands" ----------------
#
  if [catch {cli_exec $cli(fd) "enable"} result] {
    error $result $errorInfo
  }



# Example output of 'show interface GigabitEthernet0/0' that will be used to fetch interface errors
#    ...
#    Input queue: 0/75/0/0 (size/max/drops/flushes); Total output drops: 0
#    ...
#    18001 packets input, 1658315 bytes, 0 no buffer
#     Received 17984 broadcasts, 0 runts, 0 giants, 0 throttles
#     0 input errors, 0 CRC, 0 frame, 0 overrun, 0 ignored
#     0 watchdog, 0 multicast, 0 pause input
#     0 input packets with dribble condition detected
#     29872 packets output, 2390702 bytes, 0 underruns
#     0 output errors, 0 collisions, 0 interface resets
#     0 babbles, 0 late collision, 0 deferred
#     0 lost carrier, 0 no carrier, 0 pause output
#     0 output buffer failures, 0 output buffers swapped out
#

#---------------------- end of show commands ------------


#
#----------------------- "enable" ----------------------
#
  if [catch {cli_exec $cli(fd) "enable"} result] {
    error $result $errorInfo
  }


#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#
#   Fetch physical interfaces that have IP configured.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


if [catch {cli_exec $cli(fd) "sh ip int brief | exc Interface|down|\[0-9\]*/\[0-9\]*\\.\[0-9\]*
	"} result] {
    error $result $errorInfo
}
set interface $result


#--- Fetch interface counter information

for { set i 0 } { $i < [expr [llength $interface] - 1 ] } { incr i 6 } {

if [catch {cli_read_drain $cli(fd)} result] {
    error $result $errorInfo
}
if [catch {cli_read_drain $cli(fd)} result] {
    error $result $errorInfo
}

if [catch {cli_exec $cli(fd) "show interface [ lindex $interface $i ]"} result] {
    error $result $errorInfo
}
set show_interface $result

regexp {.+?([0-9]*) input errors,} $show_interface match input_errors
regexp {.+? ([0-9]*) output errors,} $show_interface match output_errors
regexp {.+? ([0-9]*) CRC} $show_interface match crc_errors
regexp {.+? ([0-9]*) interface resets} $show_interface match interface_resets
regexp {.+? Input queue: ([0-9]*)/} $show_interface match input_queue


#1111111111111111 trigger syslog & email message if input errors exceed threshold
if  {[info exists input_errors] && $input_errors > $input_errors_threshold} {
  action_syslog msg "Interface [ lindex $interface $i ] input errors threshold exceeded."  

#----------------------- send email showing input errors----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] input errors threshold exceeded! (hostname:$routername)"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "$show_interface"]
 

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#1111111111111111 end of trigger for input errors exceeding threshold

#2222222222222222 trigger syslog & email message if output errors exceed threshold
if  {[info exists input_errors] && $output_errors > $output_errors_threshold} {
  action_syslog msg "Interface [ lindex $interface $i ] output errors threshold exceeded."  

#----------------------- send email showing output errors----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] output errors threshold exceeded! (hostname:$routername"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "$show_interface"]
 

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#2222222222222222 end of trigger for output errors exceeding threshold

#3333333333333333 trigger syslog & email message if CRC errors exceed threshold
if  {[info exists input_errors] && $crc_errors > $crc_threshold} {
  action_syslog msg "Interface [ lindex $interface $i ] CRC errors threshold exceeded."  

#----------------------- send email showing CRC errors----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] CRC errors threshold exceeded! (hostname:$routername)"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "$show_interface"]
 

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#3333333333333333 end of trigger for CRC errors exceeding threshold

#4444444444444444 trigger syslog & email message if interface resets exceed threshold
if  {[info exists input_errors] && $interface_resets > $interface_reset_threshold} {
  action_syslog msg "Interface [ lindex $interface $i ] interface resets threshold exceeded."  

#----------------------- send email showing interface resets----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] interface resets threshold exceeded! (hostname:$routername"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "$show_interface"]
 

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#4444444444444444 end of trigger for interface resets exceeding threshold

#5555555555555555 trigger syslog & email message if interface is wedged (input queue > 75)
if  {[info exists input_errors] && $input_queue > 75} {
  action_syslog msg "Interface [ lindex $interface $i ] is wedged (input queue > 75)."  

if [catch {cli_exec $cli(fd) "show buffer input-interface [ lindex $interface $i ] packet"} result] {
    error $result $errorInfo
}
set show_buffer_input_interface $result

#----------------------- send email showing wedged interface (input queue > 75)----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] is wedged (input queue > 75)! (hostname:$routername"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "---------- show interface [ lindex $interface $i ]"]
  set body [format "%s\n%s" "$body" "$show_interface"]
  set body [format "%s\n\n%s" "$body" "---------- show buffer intput-interface [ lindex $interface $i ] dump"]
  set body [format "%s\n%s" "$body" "$show_buffer_input_interface"]
  
  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#5555555555555555 end of trigger for interface wedged (input queue > 75)


}
#--- end of fetch physical interface counter information loop


#++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#   Extract physical interfaces that have subinterfaces
#
#++++++++++++++++++++++++++++++++++++++++++++++++++++++

if [catch {cli_exec $cli(fd) "show ip int brief | inc \[0-9\]*/\[0-9\]*\\.\[0-9\]*"} result] {
    error $result $errorInfo
}
set subinterfaces $result

#--- Fetch interface counter information
set foobar NULL
for { set i 0 } { $i < [expr [llength $subinterfaces] - 1 ] } { incr i 6 } {

regexp {(.+?/[0-9]*)\.} [ lindex $subinterfaces $i ] match parent_interface 

if { $parent_interface != $foobar } {

if [catch {cli_read_drain $cli(fd)} result] {
    error $result $errorInfo
}
if [catch {cli_read_drain $cli(fd)} result] {
    error $result $errorInfo
}

if [catch {cli_exec $cli(fd) "show interface [ lindex $parent_interface 0]"} result] {
    error $result $errorInfo
}
set show_interface $result
set foobar $parent_interface


regexp {.+?([0-9]*) input errors,} $show_interface match input_errors
regexp {.+? ([0-9]*) output errors,} $show_interface match output_errors
regexp {.+? ([0-9]*) CRC} $show_interface match crc_errors
regexp {.+? ([0-9]*) interface resets} $show_interface match interface_resets
regexp {.+? Input queue: ([0-9]*)/} $show_interface match input_queue


#1111111111111111 trigger syslog & email message if input errors exceed threshold
if  {[info exists input_errors] && $input_errors > $input_errors_threshold} {
  action_syslog msg "Interface [ lindex $interface $i ] input errors threshold exceeded."  

#----------------------- send email showing input errors----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] input errors threshold exceeded! (hostname:$routername"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "$show_interface"]
 

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#1111111111111111 end of trigger for input errors exceeding threshold

#2222222222222222 trigger syslog & email message if output errors exceed threshold
if  {[info exists input_errors] && $output_errors > $output_errors_threshold} {
  action_syslog msg "Interface [ lindex $interface $i ] output errors threshold exceeded."  

#----------------------- send email showing output errors----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] output errors threshold exceeded! (hostname:$routername"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "$show_interface"]
 

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#2222222222222222 end of trigger for output errors exceeding threshold

#3333333333333333 trigger syslog & email message if CRC errors exceed threshold
if  {[info exists input_errors] && $crc_errors > $crc_threshold} {
  action_syslog msg "Interface [ lindex $interface $i ] CRC errors threshold exceeded."  

#----------------------- send email showing CRC errors----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] CRC errors threshold exceeded! (hostname:$routername)"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "$show_interface"]
 

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#3333333333333333 end of trigger for CRC errors exceeding threshold

#4444444444444444 trigger syslog & email message if interface resets exceed threshold
if  {[info exists input_errors] && $interface_resets > $interface_reset_threshold} {
  action_syslog msg "Interface [ lindex $interface $i ] interface resets threshold exceeded."  

#----------------------- send email showing interface resets----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] interface resets threshold exceeded! (hostname:$routername"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "$show_interface"]
 

  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#4444444444444444 end of trigger for interface resets exceeding threshold

#5555555555555555 trigger syslog & email message if interface is wedged (input queue > 75)
if  {[info exists input_errors] && $input_queue > 75} {
  action_syslog msg "Interface [ lindex $interface $i ] is wedged (input queue > 75)."  

if [catch {cli_exec $cli(fd) "show buffer input-interface [ lindex $interface $i ] packet"} result] {
    error $result $errorInfo
}
set show_buffer_input_interface $result

#----------------------- send email showing wedged interface (input queue > 75)----------------------

  action_syslog msg "Creating mail header..."
  set body [format "Mailservername: %s" "$_email_server"]
  set body [format "%s\nFrom: %s" "$body" "$_email_from"]
  set body [format "%s\nTo: %s" "$body" "$_email_to"]
  set _email_cc ""
  set body [format "%s\nCc: %s" "$body" ""]
  set body [format "%s\nSubject: %s\n" "$body" "Interface [ lindex $interface $i ] is wedged (input queue > 75)! (hostname:$routername"]
  set body [format "%s\n%s" "$body" "$routername"]
  set body [format "%s\n%s" "$body" "---------- show interface [ lindex $interface $i ]"]
  set body [format "%s\n%s" "$body" "$show_interface"]
  set body [format "%s\n\n%s" "$body" "---------- show buffer intput-interface [ lindex $interface $i ] dump"]
  set body [format "%s\n%s" "$body" "$show_buffer_input_interface"]
  
  if [catch {smtp_send_email $body} result] {
    action_syslog msg "smtp_send_email: $result"
  }

  action_syslog msg "composite_device_health_interface.tcl e-mail notification sent!"
#------------------ end of send mail --------------------
} 
#5555555555555555 end of trigger for interface wedged (input queue > 75)


}
#--- end of same physical interface check for "if { $parent_interface != $foobar }" statement


}
#--- end of fetch physical interface w/ subinterfaces counter information loop




#
#--------------------- cli close ------------------------
#
  cli_close $cli(fd) $cli(tty_id)

# eeeeeeeeeeeeeeeeeeeeeeeeeeee    End of composite_device_health_interface.tcl eeeeeeeeeeeeeeeeeeeeee
